#----------------------------------------------------------------------------------------
#
#   genreadme - Generate README.html file from template and version database
#
#   Usage: genreadme mod_name
#          genreadme mod_name source_file dest_file
#
#----------------------------------------------------------------------------------------

import os, sys, sqlite3
from jinja2 import Template
from collections import namedtuple
from BeautifulSoup import BeautifulSoup

web_root = "http://www.cosc.canterbury.ac.nz/greg.ewing/minecraft/mods/SGCraft"

def fatal(mess):
    sys.stderr.write(mess + "\n")
    sys.exit(1)

class Row(object):

    def __init__(self, cursor, row):
        d = self.__dict__
        for idx, col in enumerate(cursor.description):
            d[col[0]] = row[idx]
    
    def __str__(self):
        return str(self.__dict__)

def jinjafy(html_in):
    soup = BeautifulSoup(html_in)
    for node in soup.findAll(True, "jinja2"):
        text = " ".join(node.findAll(text = True))
        #print "jinjafy: found", text
        node.replaceWith(text)
    return str(soup)

def href(text, url):
    return '<a href="%s">%s</a>' % (url, text)

def download_href(filename):
    return href(filename, "%s/download/%s" % (web_root, filename))

def main():
    def pop_arg(default = None):
        if args:
            return args.pop(0)
        else:
            return default
    args = sys.argv[1:]
    mod_name = pop_arg()
    if not mod_name:
        fatal("Missing mod_name")
    src_filename = pop_arg("README.html")
    dst_filename = pop_arg("build/README.html")
    conn = sqlite3.connect("versions.db")
    conn.row_factory = Row
    curs = conn.cursor()
    rows = curs.execute("""select mc_version, mod_version, forge_version, forge_link, ic2_version,
        cc_version, oc_version from version order by mc_version""")
    data = rows.fetchall()
    for row in data:
        #print row
        row.jar_href = download_href("%s-%s-mc%s.jar" % (mod_name, row.mod_version, row.mc_version))
        row.doc_href = download_href("%s-%s-Doc.zip" % (mod_name, row.mod_version))
        row.src_href = download_href("%s-%s-mc%s-Source.zip" % (mod_name, row.mod_version, row.mc_version))
        row.forge_href = href(row.forge_version, row.forge_link)
    html_in = open(src_filename).read()
    html_jinja = jinjafy(html_in)
    #open("template.html", "w").write(html_jinja)
    tmpl = Template(html_jinja)
    html_out = tmpl.render(versions = data)
    open(dst_filename, "w").write(html_out)

main()
